/*
        MemView -=- Anzeige der DOS-internen Speicherbelegung

        Compiler: Microsoft C
        by Marcus Grber 1989-93

        Globale Deklarationen
*/

/*----------------------------------------------------------------------------
   Deklaration DOS-interner Tabellenstrukturen
  ----------------------------------------------------------------------------*/
#pragma pack(1)

struct MCB {            /*** Memory Control Block (MCB) */
  char ID;                      /* normalerweise 'M', 'Z' beim letzten Block */
  unsigned int PSP;             /* Zeiger auf PSP des "Besitzer-Programms" */
  unsigned int LEN;             /* Umfang des Blocks in Paragraphen */
  char x[3];
  unsigned char OWNER[8];       /* DOS 4.0, DR-DOS 5.0: Name des Besitzers */
};

struct SCB {            /*** DOS 4.0: Sub Control Block (SCB) */
  char ID;                      /* Typbezeichnung des kontrollierten Bereiches */
  unsigned int SEG;             /* Zeiger auf kontrolliertes Segment */
  unsigned int LEN;             /* Umfang des Blocks in Paragraphen */
};

struct DEVHEAD {        /*** Gertetreiber-Kopf */
  struct DEVHEAD far * LINK;    /* Zeiger auf nchstes Gert */
  unsigned int ATTR;            /* Attributwort */
  unsigned int STRAT;           /* Offset "Strategy"-Routine */
  unsigned int INTR;            /* Offset "Interrupt"-Routine */
  union {                       /* Vom Treibertyp abhngig: */
    unsigned char NAM[8];       /*   Zeichentreiber: Einheitenname */
    unsigned char NUM;          /*   Blocktreiber: Anzahl der Untereinheiten */
  } SPEC;
};
#define ATTR_CHARDEV 0x8000     /* Maske: Zeichntreiber-Bit */

struct DIB {            /*** Dos-Info-Block (zeigt auf DIB-4) */
  struct MCB far * PFAB;        /* Zeiger auf ersten MCB in Liste */
  struct DPB far * PDPB;        /* Zeiger auf ersten DPB in der Kette */
  void far * PLB;
  struct DEVHEAD far * PDDRV1;  /* Zeiger auf CLOCK-Device */
  struct DEVHEAD far * PDDRV2;  /* Zeiger auf CON-Device */
  unsigned int MAXSECT;         /* Maximaler Sektorumfang */
  union {               /* Folgende Felder verschieben sich ab DOS 3.0 */
    struct {            /*** (alt, bis DOS 2.11) */
      struct DEVHEAD NULDEV;    /* Kopf NUL-Einheit (immer erste in Kette) */
    } old;
    struct {            /*** (neu, ab DOS 3.00) */
      void far * PBUF;
      void far * PRT;
      void far * PFT;
      unsigned int x1;
      unsigned char MAXDRV;     /* Nummer der installierten Laufwerke */
      unsigned char LASTDRV;    /* Hchstzahl der installierbaren Laufwerke */
      struct DEVHEAD NULDEV;    /* Kopf NUL-Einheit (immer erste in Kette) */
    } new;
  };
};
#define DIBp(p,field) (_osmajor<3?((p)->old.field):((p)->new.field))
#define pDIBp(p,field) (_osmajor<3?&((p)->old.field):&((p)->new.field))

struct PSP {            /* Programmsegment-Vorspann */
  char QUIT[2];                 /* Sprungziel fr CP/M-komp. Programmende */
  unsigned SIZE;                /* Obergrenze programmeigenes Segment */
  char x1;
  char FKDSP[5];                /* Sprungziel fr CP/M-komp. Dispatcher */
  void far * TERMA;             /* alter INT 22h-Vektor */
  void far * CTRLC;             /* alter INT 23h-Vektor */
  void far * HRDER;             /* alter iNT 24h-Vektor */
  unsigned LDR;                 /* PSP-Segmentadresse des "Parent"-Programms */
  char DOSIND[20];              /* DOS-Handletabelle */
  unsigned ENV;                 /* Segmentadresse des Environments */
  void far * PSS;               /* Gemerkter Systemstack bei INT 21h */
  unsigned MAXFIL;              /* Umfang von DOSIND */
  void far * PDOSIN;            /* Zeiger auf DOSIND */
  char x2[18];
  char FKDSP2[2];               /* Sprungziel MS-DOS-Funktionsdispatcher */
  char x3[8];
  char FCB1[16];                /* default-FCBs */
  char FCB2[20];
  char PARLEN;                  /* Lnge der Parameterzeile */
  char PAR[127];                /* Parameterzeile */
};

struct DPB {             /*** Drive Parameter Block */
  char DRIVE;                   /* Laufwerk, das beschrieben wird */
  union {               /* Folgende Felder verschieben sich ab DOS 4.0 */
    struct {            /*** (alt, bis DOS 3.31) */
      char x1[17];
      struct DEVHEAD far *DEV;  /* Zeiger auf zugehrigen Gertetreiber-Kopf */
      char x2[2];
      struct DPB far *LINK;      /* Zeiger auf nchsten DPB */
    } old;
    struct {            /*** (neu, ab DOS 4.00) */
      char x1[18];
      struct DEVHEAD far *DEV;  /* Zeiger auf zugehrigen Gertetreiber-Kopf */
      char x2[2];
      struct DPB far *LINK;      /* Zeiger auf nchsten DPB */
    } new;
  };
};
#define DPBp(p,field) (_osmajor<4?((p)->old.field):((p)->new.field))
                                /* Zugriff auf alten/neuen DPB */

/*
 * TesSeRact data for individual TSR
 */
struct TsrParms {
    char IdCode[8];                     /* Unique TSR Identification String*/
                                        /* NOTE -- NOT NULL-TERMINATED     */
    unsigned short IdNum;               /* Unique TSR Identification Number*/
    unsigned long FuncFlags;            /* Bit map of supported functions  */
    unsigned char HotKey;               /* Hotkey used by TSR for popup    */
    unsigned char ShiftState;           /* ShiftState used by this TSR pop */
    unsigned char HotKeyFlag;           /* which hotkey is in use          */
    unsigned char ExtraHotCount;        /* Number of Extra Hotkeys to use  */
    void far * ExtraHotKeys;            /* Pointer to extra hotkeys        */
    unsigned short TsrStatus;           /* Current TSR Status Flags        */
    unsigned short TsrPSP;              /* TSR's PSP Segment               */
    void far * TsrDTA;                  /* Pointer to TSR's DTA region     */
    unsigned short TsrDSeg;             /* TSR's Default Data Segment      */
    void far * PopupStack;              /* Pointer to Popup Stack Area     */
    void far * BackStack;               /* Pointer to Background Stack     */
};

/*
 * User structure for defining additional hotkeys
 */
struct ExtraHot {
    unsigned char Hotkey;               /* hotkey to check for             */
    unsigned char ShiftState;           /* shift state for this hot key    */
    unsigned char FlagByte;             /* flag value to use               */
                                        /*  MAY NOT BE ZERO!!!             */
    };

struct ROMHeader {      /*** ROM-Erweiterungsblock (C000h-EFFFh) */
  unsigned ID;          /* Identifikation: 0xAA55 = BIOS-Erweiterung */
  unsigned char size;   /* Umfang in 512-Byte-Blcken */
  char x1[509];         /* Fller auf 512 Bytes */
};
#pragma pack()

/*----------------------------------------------------------------------------
   Programmeigene Datenstrukturen
  ----------------------------------------------------------------------------*/
struct TechInfo {       /* "Technische Info"-Struktur */
  struct TechInfo *next;        /* Nchster Infoblock */
  unsigned infotype;            /* Art der technischen Info - nach diesem
                                   Feld werden die Infos numerisch sortiert */
  union {
    struct {
      int direct;               /* 1=Eintrittspunkt in Interrupt-Kette */
    } INT;                      /*** Interrupt-Vektor */
    struct {
      struct TsrParms far *par; /* Zeiger auf TesSeRact-Parameter-Bereich */
    } TESN;                     /*** TesSeRact-Name */
    struct {
      unsigned char Shift;      /* Shift-Tasten fr Hotkey */
      unsigned char Hotkey;     /* Tastencode fr Hotkey */
    } TESH;                     /*** TesSeRact-Hotkey */
    struct {
      int files;                /* Anzahl der Dateien */
    } FILES;                    /*** Anzahl offener Dateien */
    struct {
      int state;                /* Zustand der A20-Leitung (0=aus, 1=an) */
    } A20;                      /*** A20-Zustand im HMA */
  };
};
#define INFO_MASK  0xFF00       /* Trennung "Info"-Typ von Subinfo */
#define INFO_SUB   0x00FF       /* Trennung Subinfo von "Info"-Typ */

#define INFO_INT   0x0100       /* Typ technische Info: Interrupt */
#define INFO_TESS  0x0200       /* Typ technische Info: TesSeRact-Daten */
#define INFO_TESN  0x0200       /* Technische Info: TesSeRact-Name */
#define INFO_TESH  0x0201       /* Technische Info: TesSeRact-Hotkey */
#define INFO_FILES 0x0300       /* Technische Info: Anzahl offener Dateien */
#define INFO_A20   0x0400       /* Technische Info: A20-Status im HMA */
#define INFO_ATTR  0x8000       /* Typ technische Info: Treiberattribut */

struct MemBlDesc {      /* Struktur zur Beschreibung eines Speicherblocks */
  struct MemBlDesc *next;       /* Nchster Block auf gleicher Ebene / NULL */
  struct MemBlDesc *sub;        /* Erster Unterblock / NULL */
  struct TechInfo *info;        /* Zeiger: Technische Infos zu Block / NULL */
  union {
    struct {                    /*** "Gespaltener" Zeiger auf Anfang */
      unsigned ofs;             /* Anfangsoffset */
      unsigned seg;             /* Anfangssegment */
    };
    void far *beg;              /*** Echter Zeiger auf Anfang */
  };
  long size;                    /* Umfang in Bytes */
  int nopsp;                    /* Ebene des Blocks */
  union {
    struct {                    /*** MCB-kontrollierter Block (nopsp==0) */
      unsigned psp;             /* Programmsegment-Vorspann des "Besitzers" */
      unsigned char name[9];    /* Zugehriger Dateiname, falls PSP-Block */
      int sys;                  /* MCB-Kette: 0=DOS, 1..x=andere */
    };
    struct {                    /*** Anderer Block (nopsp==1) */
      unsigned char type;       /* Blocktyp */
      union {                   /* Typenabhngige Daten */
        struct {                /*** Gertetreiber: */
          unsigned char name[9];/* Dateiname, falls DOS 4.0 */
          char mindrv,maxdrv;   /* Block: Zugeordnete Laufwerksbuchstaben */
        } D_type;
      };
    };
  };
};

#define EMS_PAGE 16384L         // Gre einer log. EMS page

struct HandleDesc {             /*** Element des programmeigenen Handle-Infos */
  unsigned hdl;                 // Handle-Wert
  char name[9];                 // EMS: Handle-Name (ASCIIZ)
  unsigned pages;               // EMS: Logische Pages zugewiesen
                                // XMS: Kilobytes zugewiesen
  char attr;                    // EMS: Attribut: 0=volatile,1=novol.,-1=keines
                                // XMS: Anzahl Locks auf dem Block
};

struct IBMINTHEAD {             /*** IBM interrupt sharing protocol entry */
  unsigned JMP;                 // jump to actual interrupt handler: 10EBh
  void far *chain;              // address of next handler in chain
  unsigned MAGIC;               // signature: 424Bh
  char EOI;                     // EOI flag
  unsigned HwReset;             // entry point for hardware reset routine
  char x[7];                    // reserved
};
#define IBMINTHEAD_JMP 0x10EB
#define IBMINTHEAD_MAGIC 0x424B

/*----------------------------------------------------------------------------*/
extern int opt_progcomp,opt_subblocks,opt_userints,opt_irets,
           opt_keep,opt_terse,opt_verbose,opt_showhard,opt_showbios,
           opt_showfree,opt_showsys,opt_showscb,opt_showother,opt_showall,
           opt_xmem;
extern unsigned opt_width;

extern int num_moreMCBs;
extern unsigned moreMCBs[];
extern int num_shownames;
extern char shownames[][9];
extern int dos4owner,dos4scb,drdosumb;
extern struct PSP far *MyPSP;
extern long MaxSize,TotalEMS,FreeEMS,DosEMS,ExtTotal,FreeXMS;
extern struct MemBlDesc *FirstBlkDesc;
extern int EMSvers,XMSvers,HMAexist;
extern struct HandleDesc HandleInfo[255];       // Info ber EMS-Handles
extern struct HandleDesc XMSHdlInfo[255];       // Info ber XMS-Handles
extern unsigned NumHandles,NumXMSHdls;          // Anzahl der EMS/XMS-Handles

/* Da ziemlich viele Pointer auerhalb des Programms zeigen,machen wir gleich:*/
#pragma check_pointer(off)
