/*
        MemView -=- Anzeige der DOS-internen Speicherbelegung

        Compiler: Microsoft C
        by Marcus Grber 1989-93

        Routinen zur Verwaltung der Blocklisten
*/

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <string.h>

#include "mv.h"
#include "mv_tool.h"


struct MemBlDesc *GetBlkInfo(struct MemBlDesc *p,void far *ptr,int level)
/*
   Zeiger auf den Datenblock zurckgeben, der an der Adresse ptr beginnt.
   Rckgabe NULL bedeutet, da kein Block an diese Adresse beginnt.
   Es werden nur maximal Blcke bis zur Ebene level bercksichtigt.
*/
{
        struct MemBlDesc *p2;


        while(p!=NULL) {                /* Bis Blockende erreicht */
          if(level>0)                   /* Auch Suche in Unter-Ebenen? */
            if((p2=GetBlkInfo(p->sub,ptr,level-1))!=NULL)
                                        /* Dann eine Ebene tiefer prfen */
              return(p2);               /* Gefunden: Dann Unterblock zurck */
          if(_phys(p->beg)==_phys(ptr)) /* Umfassender Block pat? */
            return(p);                  /* Dann zurckgeben */
          p=p->next;                    /* Jeweils einen Block weiter */
        }
        return(NULL);                   /* Gar nichts gefunden */
}

/*----------------------------------------------------------------------------*/
struct MemBlDesc *GetLocInfo(struct MemBlDesc *p,void far *ptr,int level)
/*
   Zeiger auf den Datenblock zurckgeben, der die Speicherstelle an der Adresse
   ptr enthlt. Rckgabe NULL bedeutet, da kein Block diese Adresse
   enthlt. Es werden nur maximal Blcke bis zur Ebene level bercksichtigt.
   Bei Blcken mit unbekannter Lnge (Gertetreiber!) wird geprft, ob die
   Offset-Adresse 0000h ist (Block ist Segment-justiert), und ob die Segment-
   anteile beider Adressen gleich sind (dies trifft z.B. fr Interrupts in
   Gertetreibern meist zu).
*/
{
        struct MemBlDesc *p2;
        long ph;

        ph=_phys(ptr);                  /* Adresse in physikalische */
        if(ph>0xFFFFF)                  /* Liegt in HMA? */
          return GetBlkInfo(p,(void far *)0xFFFF0000,level);
                                        /* Dann Zeiger auf HMA-Block zurck */
        while(p!=NULL) {                /* Bis Listenende */
          if(_phys(p->beg)<=ph
             && (_phys(p->beg)+p->size>ph
                 || (p->size<0
                     && FP_OFF(p->beg)==0 && FP_SEG(ptr)==FP_SEG(p->beg)))) {
                                        /* Liegt innerhalb des Blocks? */
            if(level>0 && (p2=GetLocInfo(p->sub,ptr,level-1))!=NULL)
                                        /* Evtl. eine Ebene tiefer prfen */
              return(p2);               /* Gefunden: Dann Unterblock zurck */
            return(p);                  /* Evtl. gefundenen Block zurck */
          }
          p=p->next;                    /* Jeweils einen Block weiter */
        }
        return(NULL);                   /* Nichts gefunden */
}

/*----------------------------------------------------------------------------*/
struct MemBlDesc *AddMemBlock(struct MemBlDesc **list,struct MemBlDesc *p,
                              int nopsp)
/*
   Neuen Block p in die Liste, auf deren Anfang *list zeigt, eintragen. Ist
   nopsp==1, handelt es sich um einen Block, der nicht aus der MCB-Liste
   stammt. Zurckgegeben wird ein Zeiger auf den Block in der Liste.
*/
{
        struct MemBlDesc *p1,*p2,*p3;

        p1=GetLocInfo(*list,p->beg,0);  /* Umfassenden Block suchen */
        if(p1)                          /* Block gefunden? */
          return AddMemBlock(&(p1->sub),p,nopsp);
                                        /* Dann Block in Unterebene einordnen */
        p1=*list;                       /* Zeiger auf Listenanfag */
        p3=NULL;                        /* Listenanfang hat keinen Vorgnger */
        while(p1!=NULL && _phys(p1->beg)<=_phys(p->beg)) {
                                        /* Bis Listenende od. richtiger Platz */
          if(_phys(p1->beg)+(p1->size)>_phys(p->beg))
            return p1;                  /* Zeiger zeigt in diesen Block? Dann
                                           abbrechen - schon identifiziert */
          p3=p1;                        /* "Schleppzeiger" */
          p1=p1->next;                  /* Einen Block weiter */
        }
        p2=malloc(sizeof(struct MemBlDesc));
                                        /* Platz fr Blockinfo holen */
        memcpy(p2,p,sizeof(*p));        /* Block speichern */
        if(p3==NULL)                    /* Erster Block? */
          *list=p2;                     /* Listenanfang ndern */
        else
          p3->next=p2;                  /* Nachfolger im Vorgngerblock */
        p2->next=p1;                    /* Nacfolger */
        p2->sub=NULL;                   /* Noch keine Unterblcke */
        p2->info=NULL;                  /* Keine technischen Infos */
        p2->nopsp=nopsp;                /* Blocktyp merken */
        return p2;
}

/*----------------------------------------------------------------------------*/
void AddTechInfo(struct MemBlDesc *blk,struct TechInfo *info)
/*
   Neue technische Information *info in die Informationsliste des Blocks
   *blk einsortieren.
*/
{
        struct TechInfo *p,*p1,*p2;

        p1=blk->info;                   /* Zeiger auf Listenanfag */
        p=NULL;                         /* Listenanfang hat keinen Vorgnger */
        while(p1!=NULL && p1->infotype<=info->infotype) {
                                        /* Bis Listenende od. richtiger Platz */
          p=p1;                         /* "Schleppzeiger" */
          p1=p1->next;                  /* Einen Block weiter */
        }
        p2=malloc(sizeof(struct TechInfo));
                                        /* Platz fr Blockinfo holen */
        memcpy(p2,info,sizeof(*info));  /* Block speichern */
        if(p==NULL)                     /* Erster Block? */
          blk->info=p2;                 /* Nachfolger ist alter Listenfang */
        else
          p->next=p2;                   /* Verkettung im Vorgnger */
        p2->next=p1;                    /* Nachfolger */
}

/*----------------------------------------------------------------------------*/
void MergeBlockLists(struct MemBlDesc **list1,struct MemBlDesc **list2)
/*
   Blockliste *list2 in *list1 einsortieren. Danach enthlt *list1 die
   gemeinsame Liste und *list2 die leere Liste (NULL). Es wird erwartet,
   da beide Listen in sich bereits sortiert sind.
*/
{
        struct MemBlDesc *p1,*p2,*p;

        p=NULL;                         /* keine Vorgnger in Zielliste */
        p1=*list1;                      /* Anfang der ersten Liste */
        p2=*list2;                      /* Anfang der zweiten Liste */
        do {
          while(p1!=NULL && (p2==NULL || _phys(p1->beg)<=_phys(p2->beg))) {
                                        /* Folge aus L1 einreihen */
            p=p1;                       /* Bis zum Platz fr Element aus L2 */
            p1=p1->next;                /* weitergehen (Vorgnger merken) */
          }
          while(p2!=NULL && (p1==NULL || _phys(p1->beg)>_phys(p2->beg))) {
                                        /* Folge aus L2 einreihen */
            if(p==NULL)                 /* L2 enthlt erstes Element? */
              *list1=p2;                /* Dann neuen Anfang setzen */
            else
              p->next=p2;               /* Hinter Vorgnger einreihen */
            p=p2;                       /* Neues Listenende */
            p2=p2->next;                /* Weiter in L2 */
            p->next=p1;                 /* Nachfolger liegt vorlufig in L1 */
          }
        } while(p1!=NULL && p2!=NULL);  /* Bis beide Listen durch */
        *list2=NULL;                    /* Liste 2 sicherheitshalber weg */
}

/*----------------------------------------------------------------------------*/
void MergeTechInfo(struct TechInfo **list1,struct TechInfo **list2)
/*
   Infoliste *list2 in *list1 einsortieren. Danach enthlt *list1 die
   gemeinsame Liste und *list2 die leere Liste (NULL). Es wird erwartet,
   da beide Listen in sich bereits sortiert sind.
*/
{
        struct TechInfo *p1,*p2,*p;

        p=NULL;                         /* keine Vorgnger in Zielliste */
        p1=*list1;                      /* Anfang der ersten Liste */
        p2=*list2;                      /* Anfang der zweiten Liste */
        do {
          while(p1!=NULL && (p2==NULL || p1->infotype<=p2->infotype)) {
                                        /* Folge aus L1 einreihen */
            p=p1;                       /* Bis zum Platz fr Element aus L2 */
            p1=p1->next;                /* weitergehen (Vorgnger merken) */
          }
          while(p2!=NULL && (p1==NULL || p1->infotype>p2->infotype)) {
                                        /* Folge aus L2 einreihen */
            if(p==NULL)                 /* L2 enthlt erstes Element? */
              *list1=p2;                /* Dann neuen Anfang setzen */
            else
              p->next=p2;               /* Hinter Vorgnger einreihen */
            p=p2;                       /* Neues Listenende */
            p2=p2->next;                /* Weiter in L2 */
            p->next=p1;                 /* Nachfolger liegt vorlufig in L1 */
          }
        } while(p1!=NULL && p2!=NULL);  /* Bis beide Listen durch */
        *list2=NULL;                    /* Liste 2 sicherheitshalber weg */
}

/*----------------------------------------------------------------------------*/
void MergeTechInfoList(struct TechInfo **list1,struct MemBlDesc *list2)
/*
   Infolisten aus list2 in *list1 einsortieren. Danach enthlt *list1 die
   gemeinsame Liste und list2 leere Listen (NULL). Es wird erwartet,
   da die Teillisten in sich bereits sortiert sind.
*/
{
        while(list2!=NULL) {            /* Alle Blcke durchgehen */
          MergeTechInfo(list1,&(list2->info));
                                        /* Technische Liste des Blocks */
          MergeTechInfoList(list1,list2->sub);
                                        /* Technische Listen der Unterblcke */
          list2=list2->next;            /* Weiter in der Liste */
        }
}

/*----------------------------------------------------------------------------*/
void RemoveBlocks(struct MemBlDesc **p)
/*
   Alle Blcke in der Kette ab *p und alle ihre Unterblcke lschen und aus dem
   Speicher entfernen.
*/
{
        struct MemBlDesc *p2;

        while(*p!=NULL) {               /* Solange noch Blcke da */
          RemoveBlocks(&((*p)->sub));   /* Vielleicht Unter-Unterblcke weg */
          p2=*p;                        /* Block merken */
          *p=(*p)->next;                /* Zum nchsten Block */
          free(p2);                     /* Block entfernen */
        }
}

/*----------------------------------------------------------------------------*/
void FreeBlock(unsigned seg)
/*
   MCB-Block ab Segment seg als frei betrachten und eventuell mit
   folgendem freien Block verbinden.
*/
{
        register struct MemBlDesc *p1,*p2;

        p1=GetBlkInfo(FirstBlkDesc,_ptr(seg,0x0000),0);
                                        /* Info des Blocks holen */
        p1->psp=0x0000;                 /* Block als frei markieren */
        RemoveBlocks(&(p1->sub));       /* Unterblcke entfernen */
        p2=p1->next;                    /* Zeiger auf nchsten Block */
        if(p2!=NULL && p2->psp==0x0000){/* Folgender Block ist frei? */
          p1->size+=p2->size+16;        /* Blcke werden verbunden */
          p1->next=p2->next;            /* Block in Verkettung bergehen */
          free(p2);                     /* Block im Speicher freigeben */
        }
}
